/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.alfresco.activiti.share;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.activiti.domain.idm.User;
import com.activiti.security.SecurityUtils;
import com.activiti.service.api.UserService;

@Component
public class ModelShareExtension implements Bootstrappable {
    
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    
    @Autowired
    private ModelShareWorker worker;
    
    @Autowired
    private UserService userService;

    @Value("${inteligr8.modelShareExtension.runAs:admin@app.activiti.com}")
    private String runAsUser;

    @Value("${inteligr8.modelShareExtension.enabled:true}")
    private boolean enabled;

    private boolean bootstrapped;
    
    @Override
    public void onBootstrap() {
        this.logger.trace("onBootstrap()");
        
        if (!this.enabled)
            return;
        
        this.logger.info("Model Share Extension enabled");
        this.bootstrapped = true;
        this.logger.info("Model Share Extension initialized");
    }
    
    public boolean isEnabled() {
        return this.enabled;
    }
    
    // execute every 10 minutes
    @Scheduled(
            fixedRateString = "${inteligr8.modelShareExtension.scanIntervalInMillis:600000}",
            initialDelayString = "${inteligr8.modelShareExtension.scanDelayInMillis:30000}"
    )
    private void scheduled() {
        if (!this.enabled || !this.bootstrapped)
            return;

        this.logger.trace("scheduled()");

        this.logger.debug("Assuming user: {}", this.runAsUser);
        User runAsUser = this.userService.findUserByEmail(this.runAsUser);
        SecurityUtils.assumeUser(runAsUser);
        this.logger.debug("Assumed user: {}", runAsUser.getId());
        
        this.worker.shareAllModels();
    }

}
